#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"
#include "ljackuw.h"


MODULE = LJperl     PACKAGE = LJperl        

#// ###########################################################################
#//		AISample
=head AISample

SYNOPSIS


# Example perl program which calls this module:-

use LJperl;

$idnum	    = -1;
$demo       = 0;
$stateIO    = 0;
$updateIO   = 0;
$ledOn      = 1;
@channels   = (0,1,2,3);
@gains      = (0,0,0,0);
$disableCal = 0;

$ledOn=$1 if($ARGV[0]=~/(\d+)/);
 
print "If you can see 4 status values followed by your current 8 AI voltages below, then this test has passed:\n";

my(@results)=LJperl::AISample($idnum,$demo,$stateIO,$updateIO,$ledOn,\@channels,\@gains,$disableCal);
print join("\n",@results) . "\n";
 

INFO

Reads the voltages from 1,2, or 4 analog inputs.  Also
controls/reads the 4 IO ports.  Execution time for this
function is 20 milliseconds or less.

Declaration:
long __cdecl AISample ( long *idnum,
                long demo,
                long *stateIO,
                long updateIO,
                long ledOn,
                long numChannels,
                long *channels,
                long *gains,
                long disableCal,
                long *overVoltage,
                float *voltages )

Parameter Description:
Returns:    LabJack errorcodes or 0 for no error.
Inputs:
   *idnum - Local ID, serial number, or -1 for first found.
   demo - Send 0 for normal operation, >0 for demo mode.  Demo mode allows this function to be called without a LabJack.
   *stateIO - Output states for IO0-IO3.
   updateIO - If >0, state values will be written.  Otherwise, just a read is performed.
   ledOn - If >0, the LabJack LED is turned on.
   numChannels - Number of analog input channels to read (1,2, or 4).
   *channels - Pointer to an array of channel commands with at least numChannels elements.  Each channel command is 0-7 for single-ended, or 8-11 for differential.
   *gains - Pointer to an array of gain commands with at least numChannels elements.  Gain commands are 0=1, 1=2, , 7=20.  This amplification is only available for differential channels.
   disableCal - If >0, voltages returned will be raw readings that are not corrected using calibration constants.
   *voltages - Pointer to an array where voltage readings are returned.  Send a 4-element array of zeros.
Outputs:
   *idnum - Returns the local ID or -1 if no LabJack is found.
   *overVoltage - If >0, an overvoltage has been detected on one of the selected analog inputs.

=cut

# long WINAPI AISample(long *idnum,		*
#		       long demo,		*
#		       long *stateIO,		*
#		       long updateIO,		*
#		       long ledOn,		*
#		       long numChannels,	
#		       long *channels,		*
#		       long *gains,		*
#		       long disableCal,		*
#		       long *overVoltage,	
#		       float *voltages);	

void
AISample(idnum,demo,stateIO,updateIO,ledOn,channels,gains,disableCal)
        int idnum
        int demo
        int stateIO
        int updateIO
        int ledOn
#//        int numCh
        SV * channels
        SV * gains
        int disableCal
    INIT:
        int i,n,numchannels,numgains,numCh;
        long errcode;
        long lchannels[14]; // ={0,1,2,3};
        long lgains[14];     // ={0,0,0,0};
        long ov;
        float voltages[4]={0,0,0,0};

        // Check that they passed an array of channels, and count the elements
        if ((!SvROK(channels))
            || (SvTYPE(SvRV(channels)) != SVt_PVAV)
            || ((numchannels = av_len((AV *)SvRV(channels))) < 0))
        {
            XSRETURN_UNDEF;
        }

        // Check that they passed an array of gains, and count the elements
        if ((!SvROK(gains))
            || (SvTYPE(SvRV(gains)) != SVt_PVAV)
            || ((numgains = av_len((AV *)SvRV(gains))) < 0))
        {
            XSRETURN_UNDEF;
        }

        // #Make sure there's a gain for every channel
        if(numgains<numchannels) {
            XSRETURN_UNDEF; 
        } 

    PPCODE:

        // Extract the channels we got from perl...
        for (n = 0; n <= numchannels; n++) {
                lchannels[n]= SvNV(*av_fetch((AV *)SvRV(channels), n, 0));
        }
        // Extract the gains we got from perl...
        for (n = 0; n <= numgains; n++) {
                lgains[n]= SvNV(*av_fetch((AV *)SvRV(gains), n, 0));
        }
        numCh=numchannels+1;

        // Run the command
        errcode = AISample (&idnum,demo,&stateIO,updateIO,ledOn,numCh,lchannels,lgains,disableCal,&ov,voltages);

        // Return the results to perl in a big array
        if(errcode) {
          char errorString[51];
          GetErrorString ( errcode, errorString );
          XPUSHs(sv_2mortal(newSVpv(errorString,0)));
        } else {
          XPUSHs(sv_2mortal(newSVnv(errcode)));
        }

        XPUSHs(sv_2mortal(newSVnv(idnum)));
        XPUSHs(sv_2mortal(newSVnv(ov)));
        XPUSHs(sv_2mortal(newSVnv(stateIO)));
        for(i=0;i<numCh;i++)XPUSHs(sv_2mortal(newSVnv(voltages[i])));
        
#// ###########################################################################
#//		AIBurst
=head AIBurst

SYNOPSIS

INFO

Declaration:

long _stdcall AIBurst(long *idnum,
		      long demo,
		      long stateIOin,
		      long updateIO,
		      long ledOn,
		      long numChannels,
		      long *channels,
		      long *gains,
		      float *scanRate,
		      long disableCal,
		      long triggerIO,
		      long triggerState,
		      long numScans,
		      long timeout,
		      float (*voltages)[4],
		      long *stateIOout,
		      long *overVoltage,
		      long transferMode);

AIBurst: Reads a certain number of scans at a certain scan rate
 	 from 1,2, or 4 analog inputs.  First, data is acquired and
	 stored in the LabJacks 4096 sample RAM buffer.  Then, the
	 data is transferred to the PC application.

Paramter description

	Returns:	LabJack errorcodes or 0 for no error (I32).

	Inputs:		*idnum		-Local ID, Serial Number, or -1 for first
//					 found (I32).
//			demo		-Send 0 for normal operation, >0 for demo
//					 mode (I32).  Demo mode allows this function
//					 to be called without a LabJack, and does little
//					 but simulate execution time.
//			stateIOin	-Output states for IO0-IO3 (I32).
//			updateIO	-If >0, state values will be written.  Otherwise,
//					 just reads are performed (I32).
//			ledOn		-If >0, the LabJack LED is turned on (I32).
//			numChannels	-Number of channels.  1, 2, or 4 (I32).
//			*channels	-Pointer to an array of channel commands with at
//					 least numChannels elements (I32).  Each channel
//					 command is 0-7 for SE or 8-11 for Diff.
//			*gains		-Pointer to an array of gain commands with at
//					 least numChannels elements (I32).  Gain
//					 commands are 0=1,1=2,...,7=20.  Gain only
//					 available for differential channels.
//			*scanRate	-Scans acquired per second (SGL).  A scan is a
//					 reading from every channel (1,2, or 4).  The
//					 sample rate (scanRate*numChannels) must
//					 be 400-8192.
//			disableCal	-If >0, voltages returned will be raw readings
//					 that are not corrected using calibration
//					 constants (I32).
//			triggerIO	-Set the IO port to trigger on.  0=none,
//					 1=IO0,...,4=IO3 (I32).
//			triggerState	-If >0, the acquisition will be triggered
//					 when the selected IO port reads high (I32).
//			numScans	-Number of scans which will be collected (I32).
//					 Minimum is 1.  Maximum numSamples is 4096 where
//					 numSamples is numScans * numChannels.
//			timeout		-Function timeout value in seconds (I32).
//			*voltages	-Voltage readings buffer.  Send a 4096 by 4
//					 element array of zeros (SGL).
//			*stateIOout	-IO state readings buffer.  Send a 4096 element
//					 array of zeros (I32).
//			transferMode	-0=auto,1=normal,2=turbo (I32)
//	Outputs:	*idnum		-Returns the Local ID or -1 if no LabJack is
//					 found (I32).
//			*scanRate	-Returns the actual scan rate, which due to
//					 clock resolution is not always exactly the
//					 same as the desired scan rate (SGL).
//			*voltages	-Voltage readings are returned in this 4096 by
//					 4 array (SGL). Unused locations are filled
//					 with 9999s.
//			*stateIOout	-The states of all 4 IO are returned in this
//					 array (I32).  Unused locations are filled
//					 with 9999s.
//			*overVoltage	-If >0, an overvoltage has been detected on
//					 at least one sample of at least one of the
//					 analog inputs (I32).
//
//	Time:	The execution time of this function, in milliseconds, can be
//		estimated with the below formulas.  The actual number of scans
//		collected and transferred by the LabJack is the smallest power
//		of 2 from 64 to 4096 which is at least as big as numScans.  This
//		is represented below as numScansActual.
//		Normal => 30+(1000*numScansActual/scanRate)+(2.5*numScansActual)
//		Turbo  => 30+(1000*numScansActual/scanRate)+(0.4*numScansActual)

void
AIBurst(idnum,demo,stateIOin,updateIO,ledOn,numChannels,channels,gains,scanRate,disableCal,triggerIO,triggerState,numScans,timeout,voltages,stateIOout,overVoltage,transferMode)
    long  idnum
    long  demo
    long  stateIOin
    long  updateIO
    long  ledOn
    long  numChannels
    SV *  channels
    SV *  gains
    int   scanRate
    long  disableCal
    long  triggerIO
    long  triggerState
    long  numScans
    long  timeout
    SV *  voltages
    long  stateIOout
    long  overVoltage
    long  transferMode

 INIT:
    long errcode;
 
 PPCODE:
    errcode = AIBurst(idnum, demo, stateIOin, updateIO, ledOn, numChannels,
		      channels, gains,
		      scanRate,
		      disableCal, triggerIO,
		      triggerState, numScans, timeout, voltages, stateIOout,
		      overVoltage, transferMode);

    // Return the results to perl in a big array
    if(errcode) {
	char errorString[51];
	GetErrorString ( errcode, errorString );
	XPUSHs(sv_2mortal(newSVpv(errorString,0)));
    } else {
	XPUSHs(sv_2mortal(newSVnv(errcode)));
    }
                                                                                                                                       
    XPUSHs(sv_2mortal(newSVnv(idnum)));
    XPUSHs(sv_2mortal(newSVnv(overVoltage)));
    XPUSHs(sv_2mortal(newSVnv(stateIOin)));

=cut
 
#// ###########################################################################
#//		AIStreamStart
=head AIStreamStart

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		AIStreamRead
=head AIStreamRead

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		AIStreamClear
=head AIStreamClear

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		AOUpdate
=head AOUpdate

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		AsynchConfig
=head AsynchConfig

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		Asynch
=head Asynch

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		BitsToVolts
=head BitsToVolts

SYNOPSIS

INFO

 BitsToVolts:	Converts a 12-bit (0-4095) binary value into a LabJack
		voltage.  Volts=((2*Bits*Vmax/4096)-Vmax)/Gain where
		Vmax=10 for SE, 20 for Diff.

Declaration:

long _stdcall BitsToVolts (long chnum,
			   long chgain,
			   long bits,
			   float *volts);

Paramter description
	Returns:	LabJack errorcodes or 0 for no error (I32).

	Inputs:		chnum		-Channel index.  0-7=SE, 8-11=Diff (I32).
			chgain		-Gain index.  0=1,1=2,...,7=20 (I32).
			bits		-Binary value from 0-4095 (I32).
	Outputs:	*volts		-Voltage.  SE=+/-10, Diff=+/-20 (SGL).

=cut

float
BitsToVolts(chnum, chgain, bits, volts)
    int   chnum
    int   chgain
    int   bits
    SV*  volts
  CODE:
    RETVAL = BitsToVolts(chnum, chgain, bits, volts);
  OUTPUT:
    volts
    RETVAL

#// ###########################################################################
#//		VoltsToBits
=head VoltsToBits

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

int
VoltsToBits(chnum, chgain, volts, bits)
    int   chnum
    int   chgain
    float volts
    int   bits
  CODE:
    RETVAL = BitsToVolts(chnum, chgain, volts, bits);
  OUTPUT:
    bits
    RETVAL

#// ###########################################################################
#//		Counter
=head Counter

SYNOPSIS

INFO

 Counter:	Controls and reads the counter.  The counter is disabled if
		the watchdog timer is enabled.

Declaration:
long _stdcall Counter(long *idnum,
		      long demo,
		      long *stateD,
		      long *stateIO,
		      long resetCounter,
		      long enableSTB,
		      unsigned long *count);

Paramter description

	Returns:	LabJack errorcodes or 0 for no error (I32).

	Inputs:		*idnum		-Local ID, Serial Number, or -1 for first
					 found (I32).
			demo		-Send 0 for normal operation, >0 for demo
					 mode (I32).  Demo mode allows this function to
					 be called without a LabJack, and does little but
					 simulate execution time.
			resetCounter	-If >0, the counter is reset to zero after
					 being read (I32).
			enableSTB	-If >0, STB is enabled (I32).  Only works with
					 firmware V1.02 or later.
	Outputs:	*idnum		-Returns the Local ID or -1 if no LabJack is
					 found (I32).
			*stateD		-States of D0-D15 (I32).
			*stateIO	-States of IO0-IO3 (I32).
			*count		-Current count, before reset (U32).

	Time:		20 ms

=cut

int
Counter(idnum, demo, stateD, stateIO, resetCounter, enableSTB, count)
    long * idnum
    long   demo
    long * stateD
    long * stateIO
    long   resetCounter
    long   enableSTB

    unsigned long * count

   INIT:
    int  i, j, k, n;
    long errcode;
    int  nstateD;
    long lstateD[16];
    int  nstateIO;
    long lstateIO[4];
    unsigned long *mycount;

    // Check that they passed an array of channels, and count the elements

    PPCODE:
        // Run the command
        errcode = Counter(idnum, demo, stateD, stateIO, resetCounter, enableSTB, count);
	// AISample (&idnum,demo,&stateIO,updateIO,ledOn,numCh,lchannels,lgains,disableCal,&ov,voltages);

        // Return the results to perl in a big array
        if(errcode) {
          char errorString[51];
          GetErrorString ( errcode, errorString );
          XPUSHs(sv_2mortal(newSVpv(errorString,0)));
        } else {
          XPUSHs(sv_2mortal(newSVnv(errcode)));
        }

        XPUSHs(sv_2mortal(newSVnv(idnum)));
        XPUSHs(sv_2mortal(newSVnv(stateD)));
        XPUSHs(sv_2mortal(newSVnv(stateIO)));
        XPUSHs(sv_2mortal(newSVnv(*count)));
#  CODE:
#    RETVAL = Counter(idnum, demo, stateD, stateIO, resetCounter, enableSTB, count);
#  OUTPUT:
#    idnum
#    stateD
#    stateIO
#    count
#    RETVAL

#// ###########################################################################
#//		DigitalIO
=head DigitalIO

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		EAnalogIn
=head EAnalogIn

SYNOPSIS

#use LJperl;

$idnum	    = -1;
$demo       = 0;
$stateIO    = 0;
$updateIO   = 0;
$ledOn      = 1;
@channels   = (0,1,2,3);
@gains      = (0,0,0,0);
$disableCal = 0;

INFO

Function reads the voltage from 1 analog input.  Calling
this function turns/leaves the status LED on.

Declaration:

long WINAPI EAnalogIn(long *idnum,
	              long demo,
	              long channel,
	              long gain,
	              long *overVoltage,
	              float *voltage);

Parameter Description:

Returns:	LabJack errorcodes or 0 for no error (I32).

Inputs:		*idnum		-Local ID, Serial Number, or -1 for first
				 found (I32).
		demo		-Send 0 for normal operation, >0 for demo
				 mode (I32).  Demo mode allows this function
				 to be called without a LabJack, and does
				 little but simulate execution time.
		channel		-Channel command is 0-7 for SE or 8-11 for Diff.
		gain		-Gain command is 0=1,1=2,...,7=20.  Gain only
				 available for differential channels.

Outputs:	*idnum		-Returns the Local ID or -1 if no LabJack is
				 found (I32).
		*overVoltage	-If >0, an overvoltage has been detected
				 on the analog input (I32).
		*voltage	-Returns the voltage reading (SGL).

Time:		20 ms
=cut

long
EAnalogIn(idnum, demo, channel, gain, overVoltage, voltage)
    int   idnum
    int   demo
    int   channel
    int   gain
    long  overVoltage
    float voltage
  CODE:
    RETVAL = EAnalogIn(&idnum,demo,channel,gain,&overVoltage,&voltage);
  OUTPUT:
    idnum
    overVoltage
    voltage
    RETVAL
	
#//		EAnalogOut
=head
long WINAPI EAnalogOut(long *idnum,
	               long demo,
		       float analogOut0,
		       float analogOut1);
=cut

void
EAnalogOut(idnum, demo, AOut0, AOut1)
	int idnum
	int demo
	float AOut0
	float AOut1
INIT:
        long errcode;
PPC:
	errcode = EAnalogOut(&idnum, demo, AOut0, AOut1);

        // Return the results to perl in a big array
        if(errcode) {
          char errorString[51];
          GetErrorString ( errcode, errorString );
          XPUSHs(sv_2mortal(newSVpv(errorString,0)));
        } else {
          XPUSHs(sv_2mortal(newSVnv(errcode)));
        }

#// ###########################################################################
#//		ECount
=head ECount

SYNOPSIS

INFO

Declaration:

Paramter description

=cut


#// ###########################################################################
#//		EDigitalIn
=head EDigitalIn

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		EDigitalOut
=head EDigitalOut

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		GetDriverVersion
=head GetDriverVersion

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		GetErrorString
=head GetErrorString

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		GetFirmwareVersion
=head GetFirmwareVersion

SYNOPSIS

# Example perl program which calls this module -

use LJperl;

$idnum = -1;

my (@results) = LJperl::GetFirmwareVersion($idnum);
 
print "Firware version: " . join("\n",@results) . "\n";

INFO

GetFirmwareVersion:  Used to retrieve the firmware version from
                     the LabJack''s processor.

     Returns:        Version number of the LabJack firmware or 0 for error (SGL).
     Inputs:         *idnum          -Local ID, Serial Number, or -1 for first
                                      found (I32).
     Outputs:        *idnum          -Returns the Local ID or -1 if no LabJack is
                                      found (I32).  If error, returns 512 plus
                                      a normal LabJack errorcode.

     Time:           20 ms


=cut

float
GetFirmwareVersion (idnum)
    int idnum
  CODE:
    RETVAL = GetFirmwareVersion(idnum);
  OUTPUT:
    idnum
    RETVAL

#// ###########################################################################
#//		GetWinVersion
=head GetWinVersion

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		ListAll
=head ListAll

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		LocalID
=head LocalID

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		NoThread
=head NoThread

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		PulseOut
=head PulseOut

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		PulseOutStart
=head PulseOutStart

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		PulseOutFinish
=head PulseOutFinish

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		PulseOutCalc
=head PulseOutCalc

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		ReEnum
=head ReEnum

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		Reset
=head Reset

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		ResetLJ
=head ResetLJ

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		SHT1X
=head SHT1X

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		SHTCRC
=head SHTCRC

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		SHTComm
=head SHTComm

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		Synch
=head Synch

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		Watchdog
=head Watchdog

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		ReadMem
=head ReadMem

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

#// ###########################################################################
#//		WriteMem
=head WriteMem

SYNOPSIS

INFO

Declaration:

Paramter description

=cut

